C
C-----------------------------------------------------------------------
C  FFT routines for i860 systems with tuned KAI math moutines
C-----------------------------------------------------------------------
C
      SUBROUTINE SEIFTM(N,MAG,N2)
C  RETURN FOURIER TRANSFORM MAGNITUDE AND POWER OF 2 .GE. N
C  MAX LENGTH 65536
      INTEGER N,MAG,N2
      MAG = 0
      N2 = 0
      IF (N.EQ.0) RETURN 
      N2 = 1
  10  IF (N2.GE.N .OR. N.EQ.65536) RETURN
          N2 = 2*N2
          MAG = MAG+1
          GOTO 10
      RETURN
      END
C
C-----------------------------------------------------------------------
C
      SUBROUTINE SEIFTLEN(N,PAD,NFT)
C
C  Return transform length that exceeds input length times padding
C  factor:  NFT > PAD*N
C
      INTEGER N,MAG,NFT,NP
      REAL PAD
      MAG = 0
      NFT = 0
      IF (N.EQ.0) RETURN 
      NFT = 1
      NP = NINT( REAL(N)*PAD )
  10  IF (NFT.GE.NP .OR. NP.GE.65536) RETURN
          NFT = 2*NFT
          MAG = MAG+1
          GOTO 10
      RETURN
      END
C
C-----------------------------------------------------------------------
C
      SUBROUTINE SEIMRCF( LDA, M, A, LDB, B, N2, NS, NF, SA )
C
C  Perform multiple real - to - complex forward fourier transforms
C
C  lda - leading dimension of input data to be transformed
C  m - second dimension of input/output arrays (number of transforms)
C  a(lda,m) - input real array
C  ldb - leading dimension of output complex transform array.
C  b(ldb,m) - output complex array
C  n2 - transform length (must be .ge. ns)
C  ns - number of input samples. a(ns+1:lda,1:m) is ignored.
C  nf - number of output frequencies. b(nf+1:ldb,1:m) is undefined.
C  sa - work array, length n2 + 2
C
	real a(lda,m), sa(*)
	complex b(ldb,m)
C
C  Loop over input traces
	nz = n2-ns
	do j=1,m
C  Copy to work array
		call scopy( ns, a(1,j), 1, sa, 1 )
C  Zero fill
		if (nz.gt.0) call vfill( 0.0, sa(ns+1), 1, nz )
C  Forward transform
		call seircft( n2, sa )
C  Copy to output array
		call ccopy( nf, sa, 1, b(1,j), 1 )
	enddo
C
	return
	end
C
C-----------------------------------------------------------------------
C
      SUBROUTINE SEIMCRF( LDA, M, A, LDB, B, N2, NS, NF, SA )
C
C  Perform multiple complex - to - real inverse fourier transforms
C
C  lda - leading dimension of output real inverse transform array
C  m - second dimension of input/output arrays (number of transforms)
C  a(lda,m) - output real array array
C  ldb - leading dimension of input complex transform array.
C  b(ldb,m) - input complex array
C  n2 - transform length (must be .ge. ns)
C  ns - number of output samples. a(ns+1:lda,1:m) is undefined.
C  nf - number of frequencies defined in b. b(nf+1:ldb,1:m) is ignored.
C  sa - work array, length n2 + 2
C
	real a(lda,m)
	complex b(ldb,m), sa(*)
C
C  Nyquist sample
	nyq = n2/2 + 1
	nz = 2*(nyq-nf)
C
C  Loop over input traces
	do j=1,m
C  Copy to work array
		call ccopy( nf, b(1,j), 1, sa, 1 )
C  Zero fill
		if (nz.gt.0) call vfill( 0.0, sa(nf+1), 1, nz )
C  Inverse transform
		call seicrft( n2, sa )
C  Copy to output array
		call scopy( ns, sa, 1, a(1,j), 1 )
	enddo
C
	return
	end
C
C
	subroutine seicft( n2, z, isign )
C
C  In-place forward transform, complex to complex
C
C  n2 - input transform length, must be power of 2
C  z - input/output transform array
C  isign - exponential sign, -1 for forward, +1 for inverse
C
	integer n2, isign
	complex z(*)
	include 'i860fft.inc'

	if (n2.gt.N2MAX) then
		write (*,*) 'i860fft: length too long'
		return
	endif

	if (n2.ne.kan2 .or. katype.ne.-1) then
		katype = -1
		kan2 = n2
		call cfft1d( z, n2, 0, wsave )
	endif
	
	call cfft1d( z, n2, -isign, wsave )

	return
	end
C
C
	subroutine seircft( n2, z )
C
C  In-place forward transform, real to complex
C
C  n2 - input transform length, must be power of 2
C  z - input/output transform array
C
	integer n2
	complex z(*)
	include 'i860fft.inc'

	if (n2.gt.N2MAX) then
		write (*,*) 'i860fft: length too long'
		return
	endif

	if (n2.ne.kan2 .or. katype.ne.1) then
		katype = 1
		kan2 = n2
		call scfft1d( z, n2, 0, wsave )
	endif
	
	call scfft1d( z, n2, 1, wsave )

	return
	end
C
C
	subroutine seicrft( n2, z )
C
C  In-place inverse transform, complex to real
C
C  n2 - input transform length, must be power of 2
C  z - input/output transform array, real length n2+2
C
	integer n2
	real z(*)
	include 'i860fft.inc'

	if (n2.gt.N2MAX) then
		write (*,*) 'i860fft: length too long'
		return
	endif

	if (n2.ne.kan2 .or. katype.ne.1) then
		katype = 1
		kan2 = n2
		call csfft1d( z, n2, 0, wsave )
	endif
	
	call csfft1d( z, n2, -1, wsave )

	return
	end
C
C*----------------------------------------------------------------------
C*
C*  NAME:
C*    seift2d.f - 2D Fourier transforms for real data
C*     
C*  SYNOPSIS:
C*    integer n2, m2, lda, ldb, nf, ns, ntri 
C*    real a(lda,ntri), sa(n2+2)
C*    complex b(ldb,m2)
C*
C*    call seif2rc( n2, m2, lda, a, ldb, b, nf, ns, ntri, sa )
C*    call seif2cr( n2, m2, lda, a, ldb, b, nf, ns, ntri, sa )
C*
C*  DESCRIPTION: 
C*    seif2rc performs the forward 2D fft,
C*    seif2cr performs the inverse.
C*
C*----------------------------------------------------------------------
C*
C*   HISTORY:
C*
C*      Written:
C*        12/31/91 - C. Mosher, ARCO E&PT 
C*
C*----------------------------------------------------------------------
C
	subroutine seif2rc( n2, m2, lda, a, ldb, b, nf, ns, ntri, sa )
C
C  Forward 2D Fourier Transform for real data
C
C  n2, m2 - transform lengths
C  lda - leading dimension of input array
C  a(lda,*) - input real array containing data to be transformed
C  ldb - leading dimension of output array
C  b(ldb,m2) - output complex transform
C  nf - number of frequencies to compute
C  ns - number of input samples per trace 
C  ntri - number of input traces
C  sa - work array, length n2+2
C
	implicit none
C  Argument declarations
	integer n2, m2, lda, ldb, nf, ns, ntri
	real a(lda,*), sa(*)
	complex b(*)
C  Local declarations
	integer i,j,k,nz

C  Number of points to be zeroed on input
	nz = n2 - ns

C  Loop over number of traces and forward transform
	do j=1,ntri
C  Move data to work array, pad with zeros, and transform
		call scopy( ns, a(1,j), 1, sa, 1 )
		if (nz.gt.0) call vfill( 0.0, sa(ns+1), 1, nz )
		call seircft( n2, sa )
C  Move to output array
		k = ldb*(j-1) + 1
		call ccopy( nf, sa, 1, b(k), 1 )
	enddo

C  Zero pad remainder of output array with zeros
	nz = 2*ldb*(m2-ntri)
	k = ldb*ntri + 1
	if (nz.ne.0) call vfill( 0.0, b(k), 1, nz )

C  Transpose 
	call tran21c( ldb, m2, b )

C  Loop and perform second set of transforms
	do j=1,nf
		k = m2*(j-1) + 1
		call seicft( m2, b(k), 1 )
	enddo

C  Transpose back
	call tran21c( m2, ldb, b )

	return
	end
C
C*----------------------------------------------------------------------
C
	subroutine seif2cr( n2, m2, lda, a, ldb, b, nf, ns, ntri, sa )
C
C  Inverse 2D Fourier Transform for real data
C
C  n2, m2 - transform lengths
C  lda - leading dimension of output array
C  a(lda,ntri) - output real array containing inverse transformed data
C  ldb - leading dimension of input array
C  b(ldb,m2) - input complex transform
C  nf - number of frequencies to compute
C  ns - number of output samples per trace 
C  ntri - number of output traces
C  sa - work array, length n2+2
C
	implicit none
C  Argument declarations
	integer n2, m2, lda, ldb, nf, ns, ntri
	real a(lda,*)
	complex b(*), sa(*)
C  Local declarations
	integer i,j,k,nz

C  Transpose 
	call tran21c( ldb, m2, b )

C  Loop and perform inverse transform across traces
	do j=1,nf
		k = m2*(j-1) + 1
		call seicft( m2, b(k), -1 )
	enddo

C  Transpose back
	call tran21c( m2, ldb, b )

C  Number of points to be zeroed in the frequency domain
	nz = 2*(n2/2 + 1 - nf)

C  Loop and inverse transform over samples
	do j=1,ntri
C  Move data to work array, pad with zeros, and inverse transform
		k = ldb*(j-1) + 1
		call ccopy( nf, b(k), 1, sa, 1 )
		if (nz.gt.0) call vfill( 0.0, sa(nf+1), 1, nz )
		call seicrft( n2, sa )
C  Move to output array
		call scopy( ns, sa, 1, a(1,j), 1 )
	enddo

	return
	end
C
C*----------------------------------------------------------------------
C
	subroutine seif2cc( n2, m2, idirx, idiry, lda, a, wk )
C
C  2D Fourier Transform for complex data
C
C  n2, m2 - transform lengths
C  idirx - transform direction for first FFT, 1=forward, -1=inverse
C  idiry - transform direction for second FFT, 1=forward, -1=inverse
C  lda - leading dimension of input array
C  a(lda,ntri) - input complex array containing data to be transformed
C  ldb - leading dimension of input array
C  b(ldb,m2) - output complex transform
C  nxi, nyi - input data lengths in x and y directions
C  nxo, nyo - output number of points to retain in x and y directions
C  wk - work array, length max(n2,m2)
C
	implicit none
C  Argument declarations
	integer n2, m2, idirx, idiry, lda
	complex a(lda,*), wk(*)
C  Local declarations
	integer i,j

C  Perform first transform in-place
	do j=1,m2
		call seicft( n2, a(1,j), idirx )
	enddo

C  Loop and perform second set of transforms with work array
	do i=1,n2
C  Move data to work array and transform
		call ccopy( m2, a(i,1), lda, wk, 1 )
		call seicft( m2, wk, idiry )
C  Move back to input/output array
		call ccopy( m2, wk, 1, a(i,1), lda )
	enddo

	return
	end

C*--------------------------------------------------------------------
C*  NAME:
C*    seiscopy, seiccopy - Memory to memory frame copy
C*     
C*  SYNOPSIS:
C*    integer n1a, n2a, n1b, n2b;
C*    real a(n1a,n2a), b(n1b,n2b)
C*    call seiscopy( n1a, n2a, a, n1b, n2b, b );
C*
C*  DESCRIPTION:
C*    Copies a frame from one 2D array to another, possibly in-place.
C*    Sets a(i,j) = b(i,j), truncating 'b' if destination 'a' is 
C*    smaller, zero-fills if destination is larger.
C*    'seiccopy' is the complex version.
C*
C*--------------------------------------------------------------------

	subroutine seiscopy( n1a, n2a, a, n1b, n2b, b )
	implicit none
C  Arguments
	integer n1a,n2a,n1b,n2b
	real a(n1a,n2a), b(n1b,n2b)
C  Local declarations
	integer n1, n2, i, j
	integer jloc, ja, jb

C  Set minimum copy range
	n1 = min(n1a,n1b)
	n2 = min(n2a,n2b)

C  Get start address of arrays
	ja = jloc(a)
	jb = jloc(b)

C  Back to front copy if input/output are the same and the destination
C    has a larger leading dimension
	if (ja.eq.jb .and. n1a.gt.n1b) then
		do j=n2,1,-1
			do i=n1,1,-1
				a(i,j) = b(i,j)
			enddo
		enddo
C  Otherwise, front to back copy
	else
		do j=1,n2
			do i=1,n1
				a(i,j) = b(i,j)
			enddo
		enddo
	endif
	
C  Zero pad end of rows if destination is larger
	if (n1a.gt.n1b) then
		do j=1,n2
			do i=n1+1,n1a
				a(i,j) = 0.0
			enddo
		enddo
	endif


C  Zero pad columns if destination is larger
	if (n2a .gt. n2b) then
		do j=n2b+1,n2a
			do i=1,n1a
				a(i,j) = 0.0
			enddo
		enddo
	endif

	return
	end

C  Complex argument version
	subroutine seiccopy( n1a, n2a, a, n1b, n2b, b )
	implicit none
	integer n1a, n2a, n1b, n2b
	complex a(n1a,n2a), b(n1b,n2b)
	call seiscopy( 2*n1a, n2a, a, 2*n2b, n2b, b )
	return
	end
