/*
	Seismic benchmark file I/O structure
*/

#ifndef FILE
#include <stdio.h>
#endif
#ifndef O_RDONLY
#include <fcntl.h>
#endif
#ifndef F_LOCK
#include <unistd.h>
#endif

/* Benchmark file I/O structure */
typedef struct {
	int magic;	/* magic number for valid structure */
	FILE *fp;	/* .HDR parameter file pointer */
	int tfp;	/* file pointer for binary trace data */
	int cfp;	/* file pointer for binary coordinate data */
	int afp;	/* file pointer for binary attribute data */
	int ns;		/* samples per trace */
	int ntpf;	/* traces per frame */
	int nfpv;	/* frames per volume */
	int nvol;	/* number of volumes */
	unsigned trlen;	/* trace record length in bytes */
	unsigned crlen;	/* coordinate record length in bytes */
	unsigned tflen;	/* trace frame length in bytes */
	unsigned cflen;	/* coordinate frame length in bytes */
	int vpos;	/* current volume frame position */
	int nfiles;	/* number of files making up individual datasets */
	int nfpf;	/* number of frames per file */
	int *mtfp;	/* pointer to multi-file trace data pointers */
} SBFIO;

/* Parallel Benchmark File structure */
typedef struct {
	int magic;		/* magic number for valid structure */
	SBFIO *sbf;		/* SBF structure for file on this node */
	int node;		/* Processor node number */
	int nodes;		/* Number of processing nodes */
	int ntpn;		/* Max number of traces per node */
	int ntloc;		/* Local number of traces per node */
	int nfpn;		/* Max number of frames per node */
	int nfloc;		/* Local number of frames per node */
	int nfglob;		/* Global number of frames node */
	int tmplen;		/* Required length for transpose work space */
	float *tmp;		/* Transpose work space */
} PBFIO;

/* Error number definitions */

#define SBF_MEM		1		/* memory allocation */
#define SBF_OPNH 	2		/* header file open */
#define SBF_OPNB 	3		/* binary file open */
#define SBF_CRTH 	4		/* header file create */
#define SBF_CRTB 	5		/* binary file create */
#define SBF_READ 	6		/* binary file read */
#define SBF_WRIT 	7		/* binary file write */
#define SBF_SEEK 	8		/* binary file seek */
#define SBF_INFO 	9		/* header file required info */
#define SBF_CREAT 	10		/* SBF file exists */
#define SBF_RPRM 	11		/* parameter read error */
#define SBF_WPRM 	12		/* parameter write error */
#define SBF_RANGE	13		/* function parameter range error */
#define SBF_BAD  	14		/* Bad magic number in structure */
#define SBF_LOCK 	15		/* File lock failed */
#define SBF_SIZE 	16		/* File size too large */
#define SBF_ATTR 	17		/* Attribute file error */

#define SBF_MAGIC	12345	/* Magic number id */
#define MAXCHUNK	1024*1024*512	/* max file size in words */

/* Function declarations */
SBFIO *sbf_creat(), *sbf_open();
